<?php


class DotPressAdminData
{
    private $plugin_name;
    private $version;
    /**
     * @var DotPressLoader
     */
    private $loader;

    public function __construct($plugin_name, $version, $loader)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->loader = $loader;

        $this->loader->add_action('wp_ajax_dot_press_get_placements_admin_data', $this, 'get_placements_admin_data');
        $this->loader->add_action('wp_ajax_dot_press_placement', $this, 'placements');
        $this->loader->add_action('wp_ajax_dot_press_cache_reset', $this, 'cacheReset');
    }

    public function get_placements_admin_data()
    {
        wp_send_json([
            'payload' => [
                'categories' => array_values(get_categories()),
                'tags' => array_values(get_tags())
            ],
            'status' => 'SUCCESS'
        ]);
    }

    public function placements()
    {
        $callbacks = [
            'POST' => [$this, 'modify_placements'],
            'GET' => [$this, 'get_placements'],
        ];
        $method = $_SERVER['REQUEST_METHOD'];
        $callback = $callbacks[$method];

        if (!is_null($callback) && is_callable($callback)) {
            return call_user_func($callback);
        }

        status_header(404);
        return wp_send_json([
            'status' => 'ERROR',
            'message' => 'No callback for method ' . $method
        ]);
    }

    public function modify_placements()
    {
        $callbacks = [
            'ADD' => [$this, 'add_placement'],
            'UPDATE' => [$this, 'update_placement'],
            'DELETE' => [$this, 'delete_placements']
        ];
        $operation = DotPressRequestUtils::safelyGetStringVar('operation');
        $callback = $callbacks[$operation];

        if (!is_null($callback) && is_callable($callback)) {
            return call_user_func($callback);
        }

        status_header(404);
        return wp_send_json([
            'status' => 'ERROR',
            'message' => 'No callback for operation ' . $operation
        ]);
    }

    public function get_placements()
    {
        $placements = new WP_Query([
            'post_type' => 'dot_placement',
            'post_status' => 'publish'
        ]);
        $posts = array_map(function ($post) {
            return DotPressPlacement::get_placement($post);
        }, $placements->posts);

        wp_send_json([
            'status' => 'SUCCESS',
            'payload' => $posts
        ]);
    }

    private function prepare_placement()
    {
        $widget_id = DotPressRequestUtils::safelyGetStringVar('widget_id');
        $widget_title = DotPressRequestUtils::safelyGetStringVar('widget_title');
        $insertion_point = DotPressRequestUtils::safelyGetStringVar('insertion_point');
        $insertion_number = DotPressRequestUtils::safelyGetIntVar('insertion_number');
        $alignment = DotPressRequestUtils::safelyGetStringVar('alignment');
        $format = DotPressRequestUtils::safelyGetStringVar('format');
        $tax_type = DotPressRequestUtils::safelyGetStringVar('taxonomy_type');
        $all_taxonomies = DotPressRequestUtils::ensureBooleanVar($_POST, 'all_taxonomies');

        $new_post = [
            'post_title' => sprintf('DOT Widget Placement for Widget: %s', $widget_id),
            'post_content' => '',
            'post_status' => 'publish',
            'post_type' => 'dot_placement',
            'meta_input' => [
                'widget_id' => $widget_id,
                'widget_title' => $widget_title,
                'insertion_point' => $insertion_point,
                'insertion_number' => $insertion_number,
                'alignment' => $alignment,
                'format' => $format,
                'taxonomy_type' => $tax_type,
                'all_taxonomies' => $all_taxonomies
            ]
        ];
        return $new_post;
    }

    private function ensure_placement_taxonomies($post_id, $new_post)
    {
        $tax_type = DotPressRequestUtils::safelyGetStringVar('taxonomy_type');
        $taxonomies = DotPressArrayUtils::toNumbers(explode(',', DotPressRequestUtils::safelyGetStringVar('taxonomies')));

        if ($tax_type === 'category') {
            $new_post['post_category'] = $taxonomies;
            wp_set_post_categories($post_id, $taxonomies);
        }

        if ($tax_type === 'post_tag') {
            $new_post['tags_input'] = $taxonomies;
            wp_set_post_tags($post_id, $taxonomies);
        }

        return $new_post;
    }

    private function add_update_placement($addOrUpdate)
    {
        $new_post = $this->prepare_placement();

        if ($addOrUpdate === 'UPDATE') {
            $new_post['ID'] = DotPressRequestUtils::safelyGetIntVar('placement_id');
        }

        $post_id = $addOrUpdate === 'UPDATE' ? wp_update_post($new_post) : wp_insert_post($new_post);

        if (is_wp_error($post_id)) {
            wp_send_json([
                'status' => 'ERROR',
                'message' => $post_id->get_error_message()
            ]);
            return;
        }

        $new_post = $this->ensure_placement_taxonomies($post_id, $new_post);

        wp_send_json([
            'status' => 'SUCCESS',
            'payload' => $new_post
        ]);
    }

    public function add_placement()
    {
        $this->add_update_placement('ADD');
    }

    public function update_placement()
    {
        $placement_id = DotPressRequestUtils::safelyGetIntVar('placement_id');
        $current_tax_type = DotPressArrayUtils::ensureFirst(get_post_meta($placement_id, 'taxonomy_type'));
        $new_tax_type = DotPressRequestUtils::safelyGetStringVar('taxonomy_type');

        if ($current_tax_type !== $new_tax_type) {
            wp_set_post_terms($placement_id, [], $current_tax_type);
        }

        $this->add_update_placement('UPDATE');
    }

    public function delete_placements()
    {
        $ids = DotPressArrayUtils::toNumbers(DotPressArrayUtils::ensureArray(DotPressRequestUtils::safelyGetStringVar('placement_ids')));
        $result = array_map(function ($id) {
            return wp_delete_post($id);
        }, $ids);

        wp_send_json([
            'status' => 'SUCCESS',
            'payload' => $result
        ]);
    }

    public function cacheReset()
    {
        $query = new WP_Query([
            'post_type' => 'post',
            'numberposts' => -1
        ]);

        try {
            foreach ($query->posts as $post) {
                do_action('save_post', $post->ID, $post, true);
                clean_post_cache($post);
            }
            // WordPress default cache
            if (function_exists('wp_cache_flush')) {
                wp_cache_flush();
            }

            // Purge WP-Optimize
            if (class_exists('WP_Optimize') || function_exists('WP_Optimize')) {
                WP_Optimize()->get_page_cache()->purge();
            }

            $response = function($pluginMethod) {
                return wp_send_json([
                    'status' => 'SUCCESS',
                    'message' => sprintf('Called method %s', $pluginMethod)
                ]);
            };

            // Purge plugins that use a simple method
            $others = array(
                'WP Super Cache' => 'wp_cache_clear_cache',
                'W3 Total Cache' => 'w3tc_flush_all',
                'WP Fastest Cache' => 'wpfc_clear_all_cache',
                'WP Rocket' => 'rocket_clean_domain',
                'Cachify' => 'cachify_flush_cache',
                'Comet Cache' => array('comet_cache', 'clear'),
                'SG Optimizer' => 'sg_cachepress_purge_cache',
                'Pantheon' => 'pantheon_wp_clear_edge_all',
                'Zen Cache' => array('zencache', 'clear'),
                'Breeze' => array('Breeze_PurgeCache', 'breeze_cache_flush'),
                'Swift Performance' => array('Swift_Performance_Cache', 'clear_all_cache'),
            );

            foreach ($others as $plugin => $method) {
                if (is_callable($method)) {
                    call_user_func($method);
                    return $response($method);
                }
            }

            // Purge LiteSpeed Cache
            if (is_callable(array('LiteSpeed_Cache_Tags', 'add_purge_tag'))) {
                LiteSpeed_Cache_Tags::add_purge_tag('*');
                return $response('LightSpeed');
            }

            // Purge Hyper Cache
            if (class_exists('HyperCache')) {
                do_action('autoptimize_action_cachepurged');
                return $response('HyperCache');
            }

            // Purge Godaddy Managed WordPress Hosting (Varnish + APC)
            if (class_exists('WPaaS\Plugin')) {
                self::godaddy_request('BAN');
                return $response('godaddy');
            }

            // purge cache enabler
            if (has_action('ce_clear_cache')) {
                do_action('ce_clear_cache');
                return $response('ce_clear_cache');
            }

            // Purge WP Engine
            if (class_exists("WpeCommon")) {
                if (method_exists('WpeCommon', 'purge_memcached')) {
                    WpeCommon::purge_memcached();
                }
                if (method_exists('WpeCommon', 'clear_maxcdn_cache')) {
                    WpeCommon::clear_maxcdn_cache();
                }
                if (method_exists('WpeCommon', 'purge_varnish_cache')) {
                    WpeCommon::purge_varnish_cache();
                }

                return $response('WpeCommon');
            }

            // Purge Kinsta
            global $kinsta_cache;
            if (isset($kinsta_cache) && class_exists('\\Kinsta\\CDN_Enabler')) {
                if (!empty($kinsta_cache->kinsta_cache_purge) && is_callable(array($kinsta_cache->kinsta_cache_purge, 'purge_complete_caches'))) {
                    $kinsta_cache->kinsta_cache_purge->purge_complete_caches();
                    return $response('kinsta_cache');
                }
            }

            // Purge Pagely
            if (class_exists('PagelyCachePurge')) {
                $purge_pagely = new PagelyCachePurge();
                if (is_callable(array($purge_pagely, 'purgeAll'))) {
                    $purge_pagely->purgeAll();
                    return $response('PagelyCache');
                }
            }

            // Purge Pressidum
            if (defined('WP_NINUKIS_WP_NAME') && class_exists('Ninukis_Plugin') && is_callable(array('Ninukis_Plugin', 'get_instance'))) {
                $purge_pressidum = Ninukis_Plugin::get_instance();
                if (is_callable(array($purge_pressidum, 'purgeAllCaches'))) {
                    $purge_pressidum->purgeAllCaches();
                    return $response('Pressidum');
                }
            }

            // Purge Savvii
            if (defined('\Savvii\CacheFlusherPlugin::NAME_DOMAINFLUSH_NOW')) {
                $purge_savvii = new \Savvii\CacheFlusherPlugin(); // phpcs:ignore PHPCompatibility.LanguageConstructs.NewLanguageConstructs.t_ns_separatorFound
                if (is_callable(array($purge_savvii, 'domainflush'))) {
                    $purge_savvii->domainflush();
                    return $response('Savvii');
                }
            }

            wp_send_json([
                'status' => 'SUCCESS'
            ]);
        } catch (Error $e) {
            wp_send_json([
                'status' => 'ERROR',
                'error' => $e
            ]);
        }

    }
}