<?php

class DotPressBlockRenderer
{
    private $post;
    private $categories;
    private $tags;
    private $query;
    private $posts;

    public function __construct($post)
    {
        $this->post = $post;
        $this->categories = DotPressArrayUtils::ensureArray(wp_get_post_categories($this->post->ID, ['fields' => 'ids']));
        $this->tags = DotPressArrayUtils::ensureArray(wp_get_post_tags($this->post->ID, ['fields' => 'ids']));
        $baseArgs = [
            'post_type' => 'dot_placement',
            'tax_query' => [
                'relation' => 'OR',
                [
                    'taxonomy' => 'category',
                    'terms' => $this->categories
                ],
                [
                    'taxonomy' => 'post_tag',
                    'terms' => $this->tags
                ]
            ]
        ];
        $this->query = new WP_Query($baseArgs);
        $ids = array_map(function ($post) {
            return $post->ID;
        }, $this->query->posts);
        $allQuery = new WP_Query(
            [
                'post_type' => 'dot_placement',
                'post__not_in' => $ids,
                'meta_query' => [
                    [
                        'key' => 'all_taxonomies',
                        'value' => ['1'],
                        'compare' => 'IN'
                    ],
                ]
            ]
        );

        $this->posts = array_map(function ($post) {
            return DotPressPlacement::get_placement($post);
        }, array_merge($this->query->posts, $allQuery->posts));
    }

    public function renderScript($type, $widget_id)
    {
        if ($type === 'website') {
            return sprintf('<div><div id="%s" class="dot-widget" data-url="%s/iframe/%s" data-height="410"></div><script async src="%s/dot-widget.js"></script></div>', $widget_id, DOT_PRESS_APP_URL, $widget_id, DOT_PRESS_APP_URL);
        }

        if ($type === 'amp') {
            // * NOTE - when testing on localhost, amp will strip out the tags when the src has localhost in it
            return sprintf('<div><amp-iframe src="%s/iframe/%s?iframeType=amp" height="410" sandbox="allow-scripts allow-same-origin allow-top-navigation" layout="fixed-height"><amp-img width="600" height="410" layout="fill" src="https://dotaudiences.com/dot-logo-grey.png" placeholder></amp-img></amp-iframe></div>', DOT_PRESS_APP_URL, $widget_id);
        }

        return sprintf('<div data-dot-error="type %s not found"></div>', $type);
    }

    private function getType($placement)
    {
        $insertion = $placement->meta['insertion_point'][0];

        if (preg_match('/paragraph/', $insertion)) {
            return 'core/paragraph';
        }

        if (preg_match('/image/', $insertion)) {
            return 'core/image';
        }

        return '';
    }

    private function renderAfterLast($block_type, $placement, $blocks, $currentIndex, $carry)
    {
        $type = $this->getType($placement);

        if ($type !== $block_type) {
            return $carry;
        }

        $lastIndexOf = 0;

        for ($i = 0; $i <= count($blocks); $i++) {
            if (!isset($blocks[$i])) {
                continue;
            }

            $block = $blocks[$i];
            if ($block['blockName'] === $block_type) {
                $lastIndexOf++;
            }
        }


        if ($currentIndex === $lastIndexOf) {
            return $carry . $this->renderWrapper($placement, $this->renderFormats($placement));
        }

        return $carry;
    }

    private function getBlockIndex($block, $blocks)
    {
        $blocksOfType = array_filter($blocks, function ($b) use ($block) {
            return $b['blockName'] === $block['blockName'];
        });

        return DotPressArrayUtils::findIndex(function ($b) use ($block) {
            return $b['innerHTML'] === $block['innerHTML'];
        }, $blocksOfType);
    }

    public function render($block_type, $block_content, $block)
    {
        if ($block_type) {
            if (empty($this->posts) || !isset($block['blockName'])) {
                return $block_content;
            }

            global $post;
            $blocks = parse_blocks($post->post_content);
            // * +1 as the placements positions start at 1
            $index = $this->getBlockIndex($block, $blocks) + 1;
            return array_reduce($this->posts, function ($carry, $placement) use ($block_type, $blocks, $index) {
                $possibleInsertionPoints = ['before-paragraph', 'after-paragraph', 'before-image', 'after-image'];
                $insertionPoint = $placement->meta['insertion_point'][0];
                $nonBlockPoints = ['before-content', 'after-content'];
                $insertionPointBlockTypes = [
                    'before-paragraph' => 'core/paragraph',
                    'after-paragraph' => 'core/paragraph',
                    'after-last-paragraph' => 'core/paragraph',
                    'before-image' => 'core/image',
                    'after-image' => 'core/image',
                    'after-last-image' => 'core/image',
                ];

                if (in_array($insertionPoint, $nonBlockPoints)) {
                    return $carry;
                }

                if (preg_match('/after-last/', $insertionPoint)) {
                    return $this->renderAfterLast($block_type, $placement, $blocks, $index, $carry);
                }

                $insertionPointBlockType = $insertionPointBlockTypes[$insertionPoint];

                if ($insertionPointBlockType !== $block_type) {
                    return $carry;
                }

                if (in_array($insertionPoint, $possibleInsertionPoints)) {
                    $insertionNumber = intval($placement->meta['insertion_number'][0]);

                    if ($insertionNumber === $index) {
                        $widgetTrigger = $this->renderWrapper($placement, $this->renderFormats($placement));

                        if (in_array($insertionPoint, ['before-paragraph', 'before-image'])) {
                            return $widgetTrigger . $carry;
                        }

                        if (in_array($insertionPoint, ['after-paragraph', 'after-image'])) {
                            return $carry . $widgetTrigger;
                        }
                    }
                }

                return $carry;
            }, $block_content);
        }

        return $block_content;
    }

    public function renderContentPlacements($content)
    {
        $contentPlacements = array_filter($this->posts, function ($placement) {
            $insertionPoint = $placement->meta['insertion_point'][0];
            return in_array($insertionPoint, ['before-content', 'after-content']);
        });
        $newContent = array_reduce($contentPlacements, function ($carry, $placement) {
            $widgetsContent = $this->renderWrapper($placement, $this->renderFormats($placement), false);
            if ($placement->meta['insertion_point'][0] === 'before-content') {
                return $widgetsContent . $carry;
            }

            return $carry . $widgetsContent;
        }, $content);
        return $newContent;
    }

    private function renderWrapper($placement, $carry, $includeBlockComments = true)
    {
        $marginOptions = [
            'left' => '0 auto 0 0',
            'center' => '0 auto',
            'right' => '0 0 0 auto'
        ];
        $alignment = $placement->meta['alignment'][0];
        return ($includeBlockComments ? '<!-- wp:html -->' : '') . sprintf('<div style="margin: %s; max-width: 600px; overflow: hidden;">', $marginOptions[$alignment]) . $carry . '</div>' . ($includeBlockComments ? '<!-- /wp:html -->' : '');
    }

    private function renderFormats($placement)
    {
        $widget_id = $placement->meta['widget_id'][0];
        $format = explode(',', $placement->meta['format'][0]);
        return array_reduce($format, function ($acc, $type) use ($widget_id) {
            return $acc . $this->renderScript($type, $widget_id);
        }, '');
    }
}
